@@ -18,25 +18,23 @@ Module for data store class.
 
 import os
 from shutil import copyfile
-from PySide2.QtCore import Qt, Slot
+from PySide2.QtCore import Slot
 from PySide2.QtWidgets import QAction, QFileDialog, QApplication, QMenu
-
+from spinedb_api.helpers import vacuum
 from spine_engine.project_item.project_item_resource import database_resource
 from spinetoolbox.project_item.project_item import ProjectItem
 from spinetoolbox.helpers import create_dir
 from spinetoolbox.spine_db_editor.widgets.multi_spine_db_editor import MultiSpineDBEditor
 from spine_engine.utils.serialization import serialize_path, deserialize_path
 from .commands import UpdateDSURLCommand
-from ..commands import UpdateCancelOnErrorCommand
 from .executable_item import ExecutableItem
 from .item_info import ItemInfo
-from .utils import convert_to_sqlalchemy_url
 from .output_resources import scan_for_resources
-from ..utils import database_label
+from ..utils import database_label, convert_to_sqlalchemy_url
 
 
 class DataStore(ProjectItem):
-    def __init__(self, name, description, x, y, toolbox, project, url, cancel_on_error=False):
+    def __init__(self, name, description, x, y, toolbox, project, url):
         """Data Store class.
 
         Args:
@@ -47,7 +45,6 @@ class DataStore(ProjectItem):
             toolbox (ToolboxUI): QMainWindow instance
             project (SpineToolboxProject): the project this item belongs to
             url (str or dict, optional): SQLAlchemy url
-            cancel_on_error (bool): if True, changes will be reverted on errors
         """
         super().__init__(name, description, x, y, project)
         self._toolbox = toolbox
@@ -56,13 +53,14 @@ class DataStore(ProjectItem):
             create_dir(self.logs_dir)
         except OSError:
             self._logger.msg_error.emit(f"[OSError] Creating directory {self.logs_dir} failed. Check permissions.")
-        self.cancel_on_error = cancel_on_error
         if url is None:
             url = dict()
         self._url = self.parse_url(url)
         self._multi_db_editors_open = {}
         self._open_url_menu = QMenu("Open URL in Spine DB editor", self._toolbox)
+        self._open_url_action = QAction("Open URL in Spine DB editor")
         self._open_url_menu.triggered.connect(self._handle_open_url_menu_triggered)
+        self._open_url_action.triggered.connect(self.open_url_in_spine_db_editor)
 
     @staticmethod
     def item_type():
@@ -82,8 +80,7 @@ class DataStore(ProjectItem):
         """See base class."""
         super().set_up()
         self._actions.clear()
-        self._actions.append(QAction("Open URL in Spine DB editor"))
-        self._actions[-1].triggered.connect(self.open_url_in_spine_db_editor)
+        self._actions.append(self._open_url_action)
 
     def parse_url(self, url):
         """Return a complete url dictionary from the given dict or string"""
@@ -101,25 +98,22 @@ class DataStore(ProjectItem):
         """Returns a dictionary of all shared signals and their handlers.
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
-        s[self._properties_ui.toolButton_ds_open_dir.clicked] = lambda checked=False: self.open_directory()
         s[self._properties_ui.pushButton_ds_open_editor.clicked] = self.open_url_in_spine_db_editor
-        s[self._properties_ui.toolButton_select_sqlite_file.clicked] = self.select_sqlite_file
         s[self._properties_ui.pushButton_create_new_spine_db.clicked] = self.create_new_spine_database
         s[self._properties_ui.toolButton_copy_url.clicked] = self.copy_url
+        s[self._properties_ui.toolButton_vacuum.clicked] = self.vacuum
         s[self._properties_ui.comboBox_dialect.activated[str]] = self.refresh_dialect
+        s[self._properties_ui.toolButton_select_sqlite_file.clicked] = self.select_sqlite_file
         s[self._properties_ui.lineEdit_database.file_dropped] = self.set_path_to_sqlite_file
         s[self._properties_ui.lineEdit_username.editingFinished] = self.refresh_username
         s[self._properties_ui.lineEdit_password.editingFinished] = self.refresh_password
         s[self._properties_ui.lineEdit_host.editingFinished] = self.refresh_host
         s[self._properties_ui.lineEdit_port.editingFinished] = self.refresh_port
         s[self._properties_ui.lineEdit_database.editingFinished] = self.refresh_database
-        s[self._properties_ui.cancel_on_error_checkBox.stateChanged] = self._handle_cancel_on_error_changed
         return s
 
     def restore_selections(self):
         """Load url into selections."""
-        self._properties_ui.label_ds_name.setText(self.name)
-        self._properties_ui.cancel_on_error_checkBox.setCheckState(Qt.Checked if self.cancel_on_error else Qt.Unchecked)
         self.load_url_into_selections(self._url)
 
     def url(self):
@@ -144,10 +138,17 @@ class DataStore(ProjectItem):
     def select_sqlite_file(self, checked=False):
         """Open file browser where user can select the path to an SQLite
         file that they want to use."""
+        candidate_path = os.path.abspath(self._url["database"]) if self._url["database"] else self.data_dir
+        answer = QFileDialog.getOpenFileName(self._toolbox, "Select SQLite file", candidate_path)
+        file_path = answer[0]
+        if not file_path:  # Cancel button clicked
+            return False
+        self.set_path_to_sqlite_file(file_path)
+        return True
+
+    def _new_sqlite_file(self, checked=False):
         candidate_path = os.path.abspath(os.path.join(self.data_dir, self.name + ".sqlite"))
-        answer = QFileDialog.getSaveFileName(
-            self._toolbox, "Select SQLite file", candidate_path, options=QFileDialog.DontConfirmOverwrite
-        )
+        answer = QFileDialog.getSaveFileName(self._toolbox, "Create SQLite file", candidate_path)
         file_path = answer[0]
         if not file_path:  # Cancel button clicked
             return False
@@ -155,8 +156,7 @@ class DataStore(ProjectItem):
         return True
 
     def load_url_into_selections(self, url):
-        """Load given url attribute into shared widget selections.
-        """
+        """Load given url attribute into shared widget selections."""
         if not self._active:
             return
         if not url:
@@ -168,7 +168,7 @@ class DataStore(ProjectItem):
         username = url.get("username")
         password = url.get("password")
         if dialect is not None:
-            self.enable_dialect(dialect)
+            self._properties_ui.controller.enable_dialect(dialect)
             if dialect == "":
                 self._properties_ui.comboBox_dialect.setCurrentIndex(-1)
             else:
@@ -185,6 +185,7 @@ class DataStore(ProjectItem):
             self._properties_ui.lineEdit_username.setText(username)
         if password is not None:
             self._properties_ui.lineEdit_password.setText(password)
+        self._update_actions_enabled()
 
     def update_url(self, **kwargs):
         """Set url key to value."""
@@ -195,20 +196,32 @@ class DataStore(ProjectItem):
         return True
 
     def do_update_url(self, **kwargs):
-        old_url = convert_to_sqlalchemy_url(self._url, self.name, None)
+        old_url = convert_to_sqlalchemy_url(self._url, self.name)
         self._url.update(kwargs)
-        new_url = convert_to_sqlalchemy_url(self._url, self.name, None)
+        new_url = convert_to_sqlalchemy_url(self._url, self.name)
         self.load_url_into_selections(kwargs)
         if old_url and new_url:
             old = database_resource(self.name, str(old_url), label=database_label(self.name))
             new = database_resource(self.name, str(new_url), label=database_label(self.name))
-            self._resource_to_predecessors_replaced(old, new)
-            self._resource_to_successors_replaced(old, new)
+            self._resources_to_predecessors_replaced([old], [new])
+            self._resources_to_successors_replaced([old], [new])
         else:
             self._resources_to_predecessors_changed()
             self._resources_to_successors_changed()
         self._check_notifications()
 
+    def _update_actions_enabled(self):
+        open_editor_enabled = convert_to_sqlalchemy_url(self._url, self.name) is not None
+        self._open_url_action.setEnabled(open_editor_enabled)
+        self._open_url_menu.setEnabled(open_editor_enabled)
+        if not self._active:
+            return
+        is_sqlite = self._url["dialect"].lower() == "sqlite"
+        self._properties_ui.pushButton_ds_open_editor.setEnabled(open_editor_enabled)
+        self._properties_ui.pushButton_create_new_spine_db.setEnabled(is_sqlite or open_editor_enabled)
+        self._properties_ui.toolButton_copy_url.setEnabled(open_editor_enabled)
+        self._properties_ui.toolButton_vacuum.setEnabled(is_sqlite and open_editor_enabled)
+
     @Slot()
     def refresh_host(self):
         """Refresh host from selections."""
@@ -243,98 +256,6 @@ class DataStore(ProjectItem):
     def refresh_dialect(self, dialect):
         self.update_url(dialect=dialect)
 
-    @Slot(int)
-    def _handle_cancel_on_error_changed(self, _state):
-        cancel_on_error = self._properties_ui.cancel_on_error_checkBox.isChecked()
-        if self.cancel_on_error == cancel_on_error:
-            return
-        self._toolbox.undo_stack.push(UpdateCancelOnErrorCommand(self, cancel_on_error))
-
-    def set_cancel_on_error(self, cancel_on_error):
-        self.cancel_on_error = cancel_on_error
-        if not self._active:
-            return
-        check_state = Qt.Checked if self.cancel_on_error else Qt.Unchecked
-        self._properties_ui.cancel_on_error_checkBox.blockSignals(True)
-        self._properties_ui.cancel_on_error_checkBox.setCheckState(check_state)
-        self._properties_ui.cancel_on_error_checkBox.blockSignals(False)
-
-    def enable_dialect(self, dialect):
-        """Enable the given dialect in the item controls."""
-        if dialect == "":
-            self.enable_no_dialect()
-        elif dialect == "sqlite":
-            self.enable_sqlite()
-        elif dialect == "mssql":
-            import pyodbc  # pylint: disable=import-outside-toplevel
-
-            dsns = pyodbc.dataSources()
-            # Collect dsns which use the msodbcsql driver
-            mssql_dsns = list()
-            for key, value in dsns.items():
-                if "msodbcsql" in value.lower():
-                    mssql_dsns.append(key)
-            if mssql_dsns:
-                self._properties_ui.comboBox_dsn.clear()
-                self._properties_ui.comboBox_dsn.addItems(mssql_dsns)
-                self._properties_ui.comboBox_dsn.setCurrentIndex(-1)
-                self.enable_mssql()
-            else:
-                msg = "Please create a SQL Server ODBC Data Source first."
-                self._logger.msg_warning.emit(msg)
-        else:
-            self.enable_common()
-
-    def enable_no_dialect(self):
-        """Adjust widget enabled status to default when no dialect is selected."""
-        self._properties_ui.comboBox_dialect.setEnabled(True)
-        self._properties_ui.comboBox_dsn.setEnabled(False)
-        self._properties_ui.toolButton_select_sqlite_file.setEnabled(False)
-        self._properties_ui.lineEdit_host.setEnabled(False)
-        self._properties_ui.lineEdit_port.setEnabled(False)
-        self._properties_ui.lineEdit_database.setEnabled(False)
-        self._properties_ui.lineEdit_username.setEnabled(False)
-        self._properties_ui.lineEdit_password.setEnabled(False)
-
-    def enable_mssql(self):
-        """Adjust controls to mssql connection specification."""
-        self._properties_ui.comboBox_dsn.setEnabled(True)
-        self._properties_ui.toolButton_select_sqlite_file.setEnabled(False)
-        self._properties_ui.lineEdit_host.setEnabled(False)
-        self._properties_ui.lineEdit_port.setEnabled(False)
-        self._properties_ui.lineEdit_database.setEnabled(False)
-        self._properties_ui.lineEdit_username.setEnabled(True)
-        self._properties_ui.lineEdit_password.setEnabled(True)
-        self._properties_ui.lineEdit_host.clear()
-        self._properties_ui.lineEdit_port.clear()
-        self._properties_ui.lineEdit_database.clear()
-
-    def enable_sqlite(self):
-        """Adjust controls to sqlite connection specification."""
-        self._properties_ui.comboBox_dsn.setEnabled(False)
-        self._properties_ui.comboBox_dsn.setCurrentIndex(-1)
-        self._properties_ui.toolButton_select_sqlite_file.setEnabled(True)
-        self._properties_ui.lineEdit_host.setEnabled(False)
-        self._properties_ui.lineEdit_port.setEnabled(False)
-        self._properties_ui.lineEdit_database.setEnabled(True)
-        self._properties_ui.lineEdit_username.setEnabled(False)
-        self._properties_ui.lineEdit_password.setEnabled(False)
-        self._properties_ui.lineEdit_host.clear()
-        self._properties_ui.lineEdit_port.clear()
-        self._properties_ui.lineEdit_username.clear()
-        self._properties_ui.lineEdit_password.clear()
-
-    def enable_common(self):
-        """Adjust controls to 'common' connection specification."""
-        self._properties_ui.comboBox_dsn.setEnabled(False)
-        self._properties_ui.comboBox_dsn.setCurrentIndex(-1)
-        self._properties_ui.toolButton_select_sqlite_file.setEnabled(False)
-        self._properties_ui.lineEdit_host.setEnabled(True)
-        self._properties_ui.lineEdit_port.setEnabled(True)
-        self._properties_ui.lineEdit_database.setEnabled(True)
-        self._properties_ui.lineEdit_username.setEnabled(True)
-        self._properties_ui.lineEdit_password.setEnabled(True)
-
     def actions(self):
         self._multi_db_editors_open = {x.name(): x for x in self._toolbox.db_mngr.get_all_multi_spine_db_editors()}
         if not self._multi_db_editors_open:
@@ -376,12 +297,6 @@ class DataStore(ProjectItem):
             db_editor.raise_()
             db_editor.activateWindow()
 
-    def data_files(self):
-        """Return a list of files that are in this items data directory."""
-        if not os.path.isdir(self.data_dir):
-            return None
-        return os.listdir(self.data_dir)
-
     @Slot(bool)
     def copy_url(self, checked=False):
         """Copy db url to clipboard."""
@@ -392,49 +307,31 @@ class DataStore(ProjectItem):
         QApplication.clipboard().setText(str(sa_url))
         self._logger.msg.emit(f"Database url <b>{sa_url}</b> copied to clipboard")
 
+    @Slot(bool)
+    def vacuum(self, checked=False):
+        sa_url = convert_to_sqlalchemy_url(self._url, self.name, self._logger)
+        freed, unit = vacuum(sa_url)
+        self._logger.msg.emit(f"Vacuum finished, {freed} {unit} freed from {sa_url.database}")
+
     @Slot(bool)
     def create_new_spine_database(self, checked=False):
         """Create new (empty) Spine database."""
         # Try to make an url from the current status
         sa_url = convert_to_sqlalchemy_url(self._url, self.name, self._logger)
         if not sa_url:
-            if self._url["dialect"] != "sqlite" or not self.select_sqlite_file():
+            if self._url["dialect"] != "sqlite" or not self._new_sqlite_file():
                 return
-            sa_url = convert_to_sqlalchemy_url(self._url, self.name, None)
+            sa_url = convert_to_sqlalchemy_url(self._url, self.name)
         self._toolbox.db_mngr.create_new_spine_database(sa_url, self._logger)
         self._check_notifications()
 
-    def update_name_label(self):
-        """Update Data Store tab name label. Used only when renaming project items."""
-        self._properties_ui.label_ds_name.setText(self.name)
-
-    @Slot(object, object)
-    def handle_execution_successful(self, execution_direction, engine_state):
-        """Notifies Toolbox of successful database import."""
-        if execution_direction != "FORWARD":
-            return
-        url = self.sql_alchemy_url()
-        db_map = self._toolbox.db_mngr.db_map(url)
-        if db_map is not None:
-            cookie = self
-            self._toolbox.db_mngr.session_committed.emit({db_map}, cookie)
-
     def _check_notifications(self):
         """Updates the SqlAlchemy format URL and checks for notifications"""
         self.clear_notifications()
-        if convert_to_sqlalchemy_url(self._url, self.name, logger=None) is None:
+        if convert_to_sqlalchemy_url(self._url, self.name) is None:
             self.add_notification(
                 "The URL for this Data Store is not correctly set. Set it in the Data Store Properties panel."
             )
-            return
-        db = self._url["database"]
-        if db.lower().endswith(".sqlite") and os.path.isfile(db):
-            if os.path.getsize(db) == 0:
-                self.add_notification(
-                    "This Data Store is pointing to an empty SQLITE file. "
-                    "Please click 'New Spine db' or 'Open editor...' to initialize it."
-                )
-            return
 
     def item_dict(self):
         """Returns a dictionary corresponding to this item."""
@@ -443,9 +340,13 @@ class DataStore(ProjectItem):
         # If database key is a file, change the path to relative
         if d["url"]["dialect"] == "sqlite" and d["url"]["database"]:
             d["url"]["database"] = serialize_path(d["url"]["database"], self._project.project_dir)
-        d["cancel_on_error"] = self.cancel_on_error
         return d
 
+    @staticmethod
+    def item_dict_local_entries():
+        """See base class."""
+        return [("url", "username"), ("url", "password")]
+
     def copy_local_data(self, item_dict):
         """See base class."""
         original_data_dir = item_dict.get("original_data_dir")  # (str) original dir of duplicated ProjectItem
@@ -471,7 +372,6 @@ class DataStore(ProjectItem):
             self._url["database"] = db_copy_destination
         else:
             self._url["database"] = linked_db_path
-        return
 
     @staticmethod
     def from_dict(name, item_dict, toolbox, project):
@@ -480,8 +380,7 @@ class DataStore(ProjectItem):
         url = item_dict["url"]
         if url and not isinstance(url["database"], str):
             url["database"] = deserialize_path(url["database"], project.project_dir)
-        cancel_on_error = item_dict.get("cancel_on_error", False)
-        return DataStore(name, description, x, y, toolbox, project, url, cancel_on_error)
+        return DataStore(name, description, x, y, toolbox, project, url)
 
     def rename(self, new_name, rename_data_dir_message):
         """See base class."""
@@ -490,7 +389,6 @@ class DataStore(ProjectItem):
             return False
         # If dialect is sqlite and db line edit refers to a file in the old data_dir, db line edit needs updating
         if self._url["dialect"] == "sqlite":
-            old_url = self._url["database"]
             db_dir, db_filename = os.path.split(os.path.abspath(self._url["database"].strip()))
             if db_dir == old_data_dir:
                 database = os.path.join(self.data_dir, db_filename)  # NOTE: data_dir has been updated at this point
@@ -507,25 +405,25 @@ class DataStore(ProjectItem):
                 f"Link established. Mapped data generated by <b>{source_item.name}</b> will be "
                 f"imported in <b>{self.name}</b> when executing."
             )
-        elif source_item.item_type() == "Data Store":
-            self._logger.msg.emit(
-                "Link established. "
-                f"Data from <b>{source_item.name}</b> will be merged into <b>{self.name}'s upon execution</b>."
-            )
-        elif source_item.item_type() in ["Data Connection", "Tool", "Gimlet"]:
-            # Does this type of link do anything?
+        elif source_item.item_type() in ["Tool", "Gimlet"]:
             self._logger.msg.emit("Link established")
         elif source_item.item_type() == "Data Transformer":
             self._logger.msg.emit(
                 "Link established. "
                 f"Data transformed by <b>{source_item.name}</b> will be merged into <b>{self.name}</b> upon execution."
             )
+        elif source_item.item_type() == "Merger":
+            src_ds_names = ", ".join(x.name for x in source_item.predecesor_data_stores())
+            self._logger.msg.emit(
+                "Link established. "
+                f"Data from <b>{src_ds_names}</b> will be merged into <b>{self.name}</b> upon execution."
+            )
         else:
             super().notify_destination(source_item)
 
     def resources_for_direct_successors(self):
         """See base class."""
-        sa_url = convert_to_sqlalchemy_url(self._url, self.name, None)
+        sa_url = convert_to_sqlalchemy_url(self._url, self.name)
         resources = scan_for_resources(self, sa_url)
         return resources
 
