@@ -18,8 +18,10 @@ from copy import deepcopy
 from time import monotonic
 from PySide2.QtCore import QItemSelectionModel, QModelIndex, QObject, QRunnable, Qt, QThreadPool, Signal, Slot
 from PySide2.QtWidgets import QFileDialog
+
+from spinedb_api.export_mapping.group_functions import NoGroup
 from spinedb_api.spine_io.exporters.writer import write
-from spinedb_api import DatabaseMapping
+from spinedb_api import DatabaseMapping, SpineDBVersionError, SpineDBAPIError
 from spinetoolbox.helpers import busy_effect
 from ...models import FullUrlListModel
 from ..mvcmodels.mappings_table_model import MappingsTableModel
@@ -147,6 +149,9 @@ class PreviewUpdater:
         if mappings_index.data(Qt.CheckStateRole) == Qt.Unchecked:
             return
         current_preview = self._ui.preview_tree_view.selectionModel().currentIndex()
+        if current_preview.parent().row() >= self._preview_tree_model.rowCount():
+            # May happen when mappings at the end of the list are removed.
+            return
         mapping_name = mappings_index.data()
         if current_preview.parent().data() == mapping_name:
             return
@@ -252,7 +257,7 @@ class PreviewUpdater:
         """
         if self._current_url is None:
             return
-        for row in range(first, last + 1):
+        for row in range(last, first - 1, -1):
             index = self._mappings_table_model.index(row, 0)
             if index.data(Qt.CheckStateRole) == Qt.Unchecked:
                 continue
@@ -315,7 +320,11 @@ class PreviewUpdater:
             bottom_right (QModelIndex): bottom right corner of modified mappings' in mapping list model
             roles (list of int): changed data's role
         """
-        if not {MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE, MappingsTableModel.FIXED_TABLE_NAME_ROLE} & set(roles):
+        if not {
+            MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE,
+            MappingsTableModel.FIXED_TABLE_NAME_ROLE,
+            MappingsTableModel.GROUP_FN_ROLE,
+        } & set(roles):
             return
         row = self._mappings_proxy_model.mapToSource(self._ui.mappings_table.currentIndex()).row()
         index = self._mappings_table_model.index(row, 0)
@@ -399,6 +408,7 @@ class PreviewUpdater:
             stamp,
             max_tables,
             max_rows,
+            mapping_spec.group_fn,
         )
         worker.signals.table_written.connect(self._add_or_update_data)
         self._thread_pool.start(worker)
@@ -454,7 +464,9 @@ class _Worker(QRunnable):
     class Signals(QObject):
         table_written = Signal(tuple, str, dict, float)
 
-    def __init__(self, url, mapping_name, mapping, always_export_header, stamp, max_tables=20, max_rows=20):
+    def __init__(
+        self, url, mapping_name, mapping, always_export_header, stamp, max_tables=20, max_rows=20, group_fn=NoGroup.NAME
+    ):
         super().__init__()
         self._url = url
         self._mapping_name = mapping_name
@@ -462,12 +474,22 @@ class _Worker(QRunnable):
         self._always_export_header = always_export_header
         self._max_tables = max_tables
         self._max_rows = max_rows
+        self._group_fn = group_fn
         self._stamp = stamp
         self.signals = self.Signals()
 
     @busy_effect
     def run(self):
-        db_map = DatabaseMapping(self._url)
+        try:
+            db_map = DatabaseMapping(self._url)
+        except SpineDBVersionError:
+            tables = {"error": [["unsupported database version"]]}
+            self.signals.table_written.emit((self._url, self._mapping_name), self._mapping_name, tables, self._stamp)
+            return
+        except SpineDBAPIError as error:
+            tables = {"error": [[str(error)]]}
+            self.signals.table_written.emit((self._url, self._mapping_name), self._mapping_name, tables, self._stamp)
+            return
         try:
             writer = TableWriter()
             write(
@@ -477,10 +499,15 @@ class _Worker(QRunnable):
                 empty_data_header=self._always_export_header,
                 max_tables=self._max_tables,
                 max_rows=self._max_rows,
+                group_fns=self._group_fn,
             )
             self.signals.table_written.emit(
                 (self._url, self._mapping_name), self._mapping_name, writer.tables, self._stamp
             )
+        except SpineDBAPIError as error:
+            tables = {"error": [[str(error)]]}
+            self.signals.table_written.emit((self._url, self._mapping_name), self._mapping_name, tables, self._stamp)
+            return
         finally:
             db_map.connection.close()
             self.signals.deleteLater()
