@@ -16,32 +16,26 @@ Tool class.
 :date:   19.12.2017
 """
 import os
-from PySide2.QtCore import Slot, Signal
+from PySide2.QtCore import Slot, QItemSelection, Qt
 from PySide2.QtWidgets import QAction
 from spinetoolbox.project_item.project_item import ProjectItem
 from spinetoolbox.helpers import open_url
+from spinetoolbox.mvcmodels.file_list_models import FileListModel
 from spine_engine.config import TOOL_OUTPUT_DIR
+from spine_engine.project_item.project_item_resource import CmdLineArg, cmd_line_arg_from_dict, LabelArg
 from spine_engine.utils.helpers import resolve_python_interpreter
 from .commands import UpdateToolExecuteInWorkCommand, UpdateToolOptionsCommand
-from ..commands import UpdateCmdLineArgsCommand
+from ..commands import UpdateCmdLineArgsCommand, UpdateGroupIdCommand
 from .item_info import ItemInfo
 from .widgets.custom_menus import ToolSpecificationMenu
 from .widgets.options_widgets import JuliaOptionsWidget
 from .executable_item import ExecutableItem
 from .utils import flatten_file_path_duplicates, find_file
-from ..utils import CmdLineArg, cmd_line_arg_from_dict, LabelArg
-from ..models import ToolCommandLineArgsModel, FileListModel
+from ..models import ToolCommandLineArgsModel
 from .output_resources import scan_for_resources
 
 
 class Tool(ProjectItem):
-
-    jupyter_console_requested = Signal(str, str, str)
-    persistent_console_requested = Signal(str, tuple, str)
-    persistent_stdin_available = Signal(str, str)
-    persistent_stdout_available = Signal(str, str)
-    persistent_stderr_available = Signal(str, str)
-
     def __init__(
         self,
         name,
@@ -54,6 +48,7 @@ class Tool(ProjectItem):
         execute_in_work=True,
         cmd_line_args=None,
         options=None,
+        group_id=None,
     ):
         """Tool class.
 
@@ -71,7 +66,7 @@ class Tool(ProjectItem):
         """
         super().__init__(name, description, x, y, project)
         self._toolbox = toolbox
-        self.execute_in_work = None
+        self.execute_in_work = execute_in_work
         if cmd_line_args is None:
             cmd_line_args = []
         self.cmd_line_args = cmd_line_args
@@ -79,23 +74,28 @@ class Tool(ProjectItem):
         self._specification = self._project.get_specification(specification_name)
         if specification_name and not self._specification:
             self._logger.msg_error.emit(
-                f"Tool <b>{self.name}</b> should have a Tool specification <b>{specification_name}</b> but it was not found"
+                f"Tool <b>{self.name}</b> should have a Tool specification "
+                f"<b>{specification_name}</b> but it was not found"
             )
+        self._group_id = group_id
         self._cmdline_args_model.args_updated.connect(self._push_update_cmd_line_args_command)
         self._populate_cmdline_args_model()
         self._input_file_model = FileListModel(header_label="Available resources", draggable=True)
         # Make directory for results
         self.output_dir = os.path.join(self.data_dir, TOOL_OUTPUT_DIR)
-        self.do_update_execution_mode(execute_in_work)
-        self.jupyter_console_requested.connect(self._setup_jupyter_console)
         self._specification_menu = None
         self._options = options if options is not None else {}
         self._resources_from_upstream = list()
         self._resources_from_downstream = list()
-        self.persistent_console_requested.connect(self._setup_persistent_console)
-        self.persistent_stdin_available.connect(self._add_persistent_stdin)
-        self.persistent_stdout_available.connect(self._add_persistent_stdout)
-        self.persistent_stderr_available.connect(self._add_persistent_stderr)
+
+    def set_up(self):
+        execute_in_work = self.execute_in_work
+        super().set_up()
+        self.do_update_execution_mode(execute_in_work)
+
+    @property
+    def group_id(self):
+        return self._group_id
 
     def _get_options_widget(self):
         """Returns a widget to specify the options for this tool.
@@ -132,23 +132,64 @@ class Tool(ProjectItem):
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
         s[self._properties_ui.toolButton_tool_specification.clicked] = self.show_specification_window
-        s[self._properties_ui.toolButton_tool_open_dir.clicked] = lambda checked=False: self.open_directory()
         s[self._properties_ui.pushButton_tool_results.clicked] = self._open_results_directory
         s[self._properties_ui.comboBox_tool.textActivated] = self.update_specification
         s[self._properties_ui.radioButton_execute_in_work.toggled] = self.update_execution_mode
         s[self._properties_ui.toolButton_add_file_path_arg.clicked] = self._add_selected_file_path_args
         s[self._properties_ui.toolButton_remove_arg.clicked] = self._remove_arg
+        s[
+            self._properties_ui.treeView_input_files.selectionModel().selectionChanged
+        ] = self._update_add_args_button_enabled
+        s[
+            self._properties_ui.treeView_cmdline_args.selectionModel().selectionChanged
+        ] = self._update_remove_args_button_enabled
+        s[self._properties_ui.lineEdit_group_id.editingFinished] = self._set_group_id
         return s
 
+    @Slot()
+    def _set_group_id(self):
+        """Pushes a command to update group id whenever the user edits the line edit."""
+        group_id = self._properties_ui.lineEdit_group_id.text()
+        if not group_id:
+            group_id = None
+        if self._group_id == group_id:
+            return
+        self._toolbox.undo_stack.push(UpdateGroupIdCommand(self, group_id))
+
+    def do_set_group_id(self, group_id):
+        """Sets group id."""
+        self._group_id = group_id
+        if self._active:
+            self._properties_ui.lineEdit_group_id.setText(group_id)
+
+    @Slot(QItemSelection, QItemSelection)
+    def _update_add_args_button_enabled(self, _selected, _deselected):
+        self._do_update_add_args_button_enabled()
+
+    def _do_update_add_args_button_enabled(self):
+        enabled = self._properties_ui.treeView_input_files.selectionModel().hasSelection()
+        self._properties_ui.toolButton_add_file_path_arg.setEnabled(enabled)
+
+    @Slot(QItemSelection, QItemSelection)
+    def _update_remove_args_button_enabled(self, _selected, _deselected):
+        self._do_update_remove_args_button_enabled()
+
+    def _do_update_remove_args_button_enabled(self):
+        enabled = self._properties_ui.treeView_cmdline_args.selectionModel().hasSelection()
+        self._properties_ui.toolButton_remove_arg.setEnabled(enabled)
+
     def restore_selections(self):
         """Restore selections into shared widgets when this project item is selected."""
-        self._properties_ui.label_tool_name.setText(self.name)
         self._properties_ui.treeView_input_files.setModel(self._input_file_model)
         self._properties_ui.treeView_cmdline_args.setModel(self._cmdline_args_model)
         self._properties_ui.treeView_cmdline_args.expandAll()
         self.update_execute_in_work_button()
-        self._properties_ui.label_jupyter.setText("")
+        self._properties_ui.label_jupyter.elided_mode = Qt.ElideMiddle
+        self._properties_ui.label_jupyter.hide()
         self._update_tool_ui()
+        self._do_update_add_args_button_enabled()
+        self._do_update_remove_args_button_enabled()
+        self._properties_ui.lineEdit_group_id.setText(self._group_id)
 
     @Slot(bool)
     def show_specification_window(self, _=True):
@@ -173,11 +214,13 @@ class Tool(ProjectItem):
         if not self._active:
             return
         self._properties_ui.radioButton_execute_in_work.blockSignals(True)
+        self._properties_ui.radioButton_execute_in_source.blockSignals(True)
         if self.execute_in_work:
             self._properties_ui.radioButton_execute_in_work.setChecked(True)
         else:
             self._properties_ui.radioButton_execute_in_source.setChecked(True)
         self._properties_ui.radioButton_execute_in_work.blockSignals(False)
+        self._properties_ui.radioButton_execute_in_source.blockSignals(False)
 
     @Slot(str)
     def update_specification(self, text):
@@ -258,8 +301,7 @@ class Tool(ProjectItem):
             _ = self._get_options_widget()
 
     def _update_tool_ui(self):
-        """Updates Tool properties UI. Used when Tool specification is changed.
-        Overrides execution mode (work or source) with the specification default."""
+        """Updates Tool properties UI. Used when Tool specification is changed.."""
         options_widget = self._properties_ui.horizontalLayout_options.takeAt(0)
         if options_widget:
             options_widget.widget().hide()
@@ -267,26 +309,27 @@ class Tool(ProjectItem):
             self._properties_ui.comboBox_tool.setCurrentIndex(-1)
             self.do_update_execution_mode(True)
             self._properties_ui.toolButton_tool_specification.setMenu(None)
-        else:
-            self._properties_ui.comboBox_tool.setCurrentText(self.specification().name)
-            self._update_specification_menu()
-            self._properties_ui.toolButton_tool_specification.setMenu(self._specification_menu)
-            options_widget = self._get_options_widget()
-            if options_widget:
-                self._properties_ui.horizontalLayout_options.addWidget(options_widget)
-                options_widget.show()
-            if self._specification.tooltype == "python":
-                self.specification().set_execution_settings()
-                k_spec_name = self.specification().execution_settings["kernel_spec_name"]
-                env = self.specification().execution_settings["env"]
-                use_console = self.specification().execution_settings["use_jupyter_console"]
-                if not use_console:
-                    exe = self.specification().execution_settings["executable"]
-                    p = resolve_python_interpreter(exe)
-                    self._properties_ui.label_jupyter.setText(f"[Python console] {p}")
-                else:
-                    env = "" if not env else f"[{env}]"
-                    self._properties_ui.label_jupyter.setText(f"[Jupyter Console] {k_spec_name} {env}")
+            return
+        self._properties_ui.comboBox_tool.setCurrentText(self.specification().name)
+        self._update_specification_menu()
+        self._properties_ui.toolButton_tool_specification.setMenu(self._specification_menu)
+        options_widget = self._get_options_widget()
+        if options_widget:
+            self._properties_ui.horizontalLayout_options.addWidget(options_widget)
+            options_widget.show()
+        if self._specification.tooltype == "python":
+            self.specification().set_execution_settings()
+            k_spec_name = self.specification().execution_settings["kernel_spec_name"]
+            env = self.specification().execution_settings["env"]
+            use_console = self.specification().execution_settings["use_jupyter_console"]
+            self._properties_ui.label_jupyter.show()
+            if not use_console:
+                exe = self.specification().execution_settings["executable"]
+                p = resolve_python_interpreter(exe)
+                self._properties_ui.label_jupyter.setText(f"[Basic console] {p}")
+            else:
+                env = "" if not env else f"[{env}]"
+                self._properties_ui.label_jupyter.setText(f"[Jupyter Console] {k_spec_name} {env}")
 
     def _update_specification_menu(self):
         spec_model_index = self._toolbox.specification_model.specification_index(self.specification().name)
@@ -309,10 +352,6 @@ class Tool(ProjectItem):
         """Returns Tool specification."""
         return self._specification
 
-    def update_name_label(self):
-        """Update Tool tab name label. Used only when renaming project items."""
-        self._properties_ui.label_tool_name.setText(self.name)
-
     def resources_for_direct_successors(self):
         """See base class"""
         return scan_for_resources(self, self.specification(), self.output_dir)
@@ -347,7 +386,7 @@ class Tool(ProjectItem):
         self.clear_notifications()
         if not self.specification():
             self.add_notification("This Tool does not have a specification. Set it in the Tool Properties Panel.")
-        elif not self.specification().path:
+        elif self.specification().includes and not self.specification().path:
             n = self.specification().name
             self.add_notification(
                 f"Tool specification <b>{n}</b> path does not exist. Fix this in Tool specification editor."
@@ -380,7 +419,7 @@ class Tool(ProjectItem):
         self._resources_from_upstream = resources
         self._update_files_and_cmd_line_args()
 
-    def replace_resource_from_upstream(self, old, new):
+    def replace_resources_from_upstream(self, old, new):
         """See base class."""
         self._replace_resources(old, new, self._resources_from_upstream)
 
@@ -389,48 +428,50 @@ class Tool(ProjectItem):
         self._resources_from_downstream = resources
         self._update_files_and_cmd_line_args()
 
-    def replace_resource_from_downstream(self, old, new):
+    def replace_resources_from_downstream(self, old, new):
         """See base class."""
         self._replace_resources(old, new, self._resources_from_downstream)
 
-    def _replace_resources(self, old, new, resource_list):
+    def _replace_resources(self, old_resources, new_resources, resource_list):
         """Replaces resources.
 
         Modifies ``resource_list`` in-place!
 
         Args:
-            old (ProjectItemResource): old resource
-            new (ProjectItemResource): new resource
+            old_resources (list of ProjectItemResource): old resources
+            new_resources (list of ProjectItemResource): new resources
             resource_list (list of ProjectItemResource): current downstream or upstream resources
         """
-        updated_resources = list()
-        for resource in resource_list:
-            if resource == old:
-                updated_resources.append(new)
-            else:
-                updated_resources.append(resource)
-        resource_list.clear()
-        resource_list += updated_resources
+        for old, new in zip(old_resources, new_resources):
+            for i, resource in enumerate(resource_list):
+                if resource == old:
+                    resource_list[i] = new
+                    break
+            for i, arg in enumerate(self.cmd_line_args):
+                if arg.arg == old.label:
+                    self.cmd_line_args[i] = LabelArg(new.label)
         self._input_file_model.update(self._resources_from_upstream + self._resources_from_downstream)
-        self._check_notifications()
-        cmd_line_args = list()
-        for arg in self.cmd_line_args:
-            if arg.arg == old.label:
-                cmd_line_args.append(LabelArg(new.label))
-            else:
-                cmd_line_args.append(arg)
-        self.update_cmd_line_args(cmd_line_args)
+        self.update_cmd_line_args(self.cmd_line_args)
 
     def _update_files_and_cmd_line_args(self):
         """Updates the file model and command line arguments."""
         resources = self._resources_from_upstream + self._resources_from_downstream
         self._input_file_model.update(resources)
-        self._check_notifications()
         update_args = list()
         resource_labels = {resource.label for resource in resources}
         for arg in self.cmd_line_args:
-            if isinstance(arg, LabelArg):
-                arg.missing = arg.arg not in resource_labels
+            if arg.arg in resource_labels:
+                arg.missing = False
+            elif isinstance(arg, LabelArg):
+                # Arg may refer to legacy Data connection resource labels that weren't prefixed by <project> or <data>
+                legacy_arg_converted = False
+                for label in resource_labels:
+                    if label.startswith("<") and label.endswith(arg.arg):
+                        legacy_arg_converted = True
+                        arg.arg = label
+                        break
+                if not legacy_arg_converted:
+                    arg.missing = True
             update_args.append(arg)
         self.update_cmd_line_args(update_args)
 
@@ -445,6 +486,8 @@ class Tool(ProjectItem):
         d["cmd_line_args"] = [arg.to_dict() for arg in self.cmd_line_args]
         if self._options:
             d["options"] = self._options
+        if self._group_id:
+            d["group_id"] = self._group_id
         return d
 
     @staticmethod
@@ -456,8 +499,19 @@ class Tool(ProjectItem):
         cmd_line_args = item_dict.get("cmd_line_args", [])
         cmd_line_args = [cmd_line_arg_from_dict(arg) for arg in cmd_line_args]
         options = item_dict.get("options", {})
+        group_id = item_dict.get("group_id")
         return Tool(
-            name, description, x, y, toolbox, project, specification_name, execute_in_work, cmd_line_args, options
+            name,
+            description,
+            x,
+            y,
+            toolbox,
+            project,
+            specification_name,
+            execute_in_work,
+            cmd_line_args,
+            options,
+            group_id,
         )
 
     def rename(self, new_name, rename_data_dir_message):
@@ -465,7 +519,6 @@ class Tool(ProjectItem):
         if not super().rename(new_name, rename_data_dir_message):
             return False
         self.output_dir = os.path.join(self.data_dir, TOOL_OUTPUT_DIR)
-        self._resources_to_successors_changed()
         return True
 
     def notify_destination(self, source_item):
@@ -495,57 +548,6 @@ class Tool(ProjectItem):
         else:
             super().notify_destination(source_item)
 
-    @Slot(str, str, str)
-    def _setup_jupyter_console(self, filter_id, kernel_name, connection_file):
-        """Sets up jupyter console, eventually for a filter execution.
-
-        Args:
-            filter_id (str): filter identifier
-            kernel_name (str): jupyter kernel name
-            connection_file (str): path to connection file
-        """
-        if not filter_id:
-            # pylint: disable=attribute-defined-outside-init
-            self.console = self._toolbox.make_jupyter_console(self, kernel_name, connection_file)
-            self._toolbox.override_console()
-        else:
-            self._filter_consoles[filter_id] = self._toolbox.make_jupyter_console(self, kernel_name, connection_file)
-            self._toolbox.ui.listView_executions.model().layoutChanged.emit()
-
-    @Slot(str, tuple, str)
-    def _setup_persistent_console(self, filter_id, key, language):
-        """Sets up persistent console, eventually for a filter execution.
-
-        Args:
-            filter_id (str): filter identifier
-            key (tuple)
-            language (str)
-        """
-        if not filter_id:
-            # pylint: disable=attribute-defined-outside-init
-            self.console = self._toolbox.make_persistent_console(self, key, language)
-            self._toolbox.override_console()
-        else:
-            self._filter_consoles[filter_id] = self._toolbox.make_persistent_console(self, key, language)
-            self._toolbox.ui.listView_executions.model().layoutChanged.emit()
-
-    @Slot(str, str)
-    def _add_persistent_stdin(self, filter_id, data):
-        self._get_console(filter_id).add_stdin(data)
-
-    @Slot(str, str)
-    def _add_persistent_stdout(self, filter_id, data):
-        self._get_console(filter_id).add_stdout(data)
-
-    @Slot(str, str)
-    def _add_persistent_stderr(self, filter_id, data):
-        self._get_console(filter_id).add_stderr(data)
-
-    def _get_console(self, filter_id):
-        if not filter_id:
-            return self.console
-        return self._filter_consoles.get(filter_id)
-
     def actions(self):
         # pylint: disable=attribute-defined-outside-init
         if self.specification() is not None:
