@@ -46,7 +46,7 @@ LIST_REQUIRED_KEYS = ["includes", "inputfiles", "inputfiles_opt", "outputfiles"]
 
 
 def make_specification(definition, app_settings, logger):
-    """De-serializes and constructs a tool specification from definition.
+    """Deserializes and constructs a tool specification from definition.
 
     Args:
         definition (dict): a dictionary containing the serialized specification.
@@ -57,11 +57,16 @@ def make_specification(definition, app_settings, logger):
         ToolSpecification: a tool specification constructed from the given definition,
             or None if there was an error
     """
-    definition["includes_main_path"] = path = definition.setdefault("includes_main_path", ".").replace("/", os.path.sep)
-    if not os.path.isabs(path):
-        definition_file_path = definition["definition_file_path"]
-        path = os.path.normpath(os.path.join(os.path.dirname(definition_file_path), path))
-    definition["includes"] = [src_file.replace("/", os.path.sep) for src_file in definition["includes"]]
+    if not definition["includes_main_path"]:
+        path = None
+    else:
+        definition["includes_main_path"] = path = definition.setdefault("includes_main_path", ".").replace(
+            "/", os.path.sep
+        )
+        if not os.path.isabs(path):
+            definition_file_path = definition["definition_file_path"]
+            path = os.path.normpath(os.path.join(os.path.dirname(definition_file_path), path))
+        definition["includes"] = [src_file.replace("/", os.path.sep) for src_file in definition["includes"]]
     try:
         _tooltype = definition["tooltype"].lower()
     except KeyError:
@@ -106,7 +111,7 @@ class ToolSpecification(ProjectItemSpecification):
 
         Args:
             name (str): Tool specification name
-            tooltype (str): Type of Tool (e.g. Python, Julia, ..)
+            tooltype (str): Type of Tool Specification (e.g. Python, Julia, ..)
             path (str): Path to Tool specification
             includes (list): List of files belonging to the tool specification (relative to 'path')
             settings (QSettings): Toolbox settings
@@ -161,7 +166,7 @@ class ToolSpecification(ProjectItemSpecification):
         }
 
     def set_execution_settings(self):
-        raise NotImplementedError
+        """Updates Tool specifications by adding the default execution settings dict for this specification."""
 
     def save(self):
         """See base class."""
@@ -206,7 +211,7 @@ class ToolSpecification(ProjectItemSpecification):
             logger (LoggerInterface): A logger instance
 
         Returns:
-            Dictionary or None if there was a problem in the tool definition.
+            dict: definition or None if there was a problem in the tool definition.
         """
         kwargs = dict()
         for p in REQUIRED_KEYS + OPTIONAL_KEYS:
@@ -230,15 +235,11 @@ class ToolSpecification(ProjectItemSpecification):
         """Returns an instance of this tool specification that is configured to run in the given directory.
 
         Args:
-            basedir (str): the path to the directory where the instance will run
-            logger (LoggerInterface)
-            owner (ExecutableItemBase): The item that owns the instance
+            basedir (str): Path to the directory where the instance will run
+            logger (LoggerInterface): Logger
+            owner (ExecutableItemBase): Project item that owns the instance
         """
-        return self.tool_instance_factory(self, basedir, self._settings, logger, owner)
-
-    @property
-    def tool_instance_factory(self):
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class GAMSTool(ToolSpecification):
@@ -265,10 +266,10 @@ class GAMSTool(ToolSpecification):
             name (str): GAMS Tool name
             tooltype (str): Tool specification type
             path (str): Path to model main file
-            includes (list): List of files belonging to the tool (relative to 'path').  # TODO: Change to src_files
+            includes (list): List of files belonging to the tool (relative to 'path').
+                First file in the list is the main GAMS program.
             settings (QSettings): Toolbox settings
             logger (LoggerInterface): a logger instance
-            First file in the list is the main GAMS program.
             description (str): GAMS Tool description
             inputfiles (list): List of required data files
             inputfiles_opt (list, optional): List of optional data files (wildcards may be used)
@@ -293,10 +294,7 @@ class GAMSTool(ToolSpecification):
         # Add .lst file to list of output files
         self.lst_file = os.path.splitext(main_file)[0] + ".lst"
         self.outputfiles.add(self.lst_file)
-        # Split main_prgm to main_dir and main_prgm
-        # because GAMS needs to run in the directory of the main program
-        # TODO: This does not work because main_file is always just file name
-        self.main_dir, self.main_prgm = os.path.split(main_file)
+        self.main_prgm = main_file
         self.gams_options = OrderedDict()
         self.return_codes = {
             0: "Normal return",
@@ -329,6 +327,58 @@ class GAMSTool(ToolSpecification):
         else:
             logging.error("Updating GAMS options failed. Unknown key: %s", key)
 
+    def make_debug_project(self, basedir):
+        """Make a GAMS project file and return an anchor for the execution item log.
+
+        Args:
+            basedir (str): Full path to work or source dir depending on execution mode
+
+        Returns:
+            Anchor to gamside project anchor or None if operation failed
+        """
+        prj_file_path = os.path.abspath(os.path.join(self.path, self.short_name + "_autocreated.gpr"))
+        try:
+            self.make_gamside_project_file(prj_file_path, basedir)
+        except OSError:
+            return None
+        anchor = (
+            "<a style='color:#99CCFF;' title='"
+            + prj_file_path
+            + "' href='file:///"
+            + prj_file_path
+            + "'>Click here to debug in GAMSIDE</a>"
+        )
+        return anchor
+
+    def make_gamside_project_file(self, prj_file_path, basedir):
+        """Make a GAMSIDE project file for debugging.
+
+        Args:
+            prj_file_path (str): Full path to GAMSIDE project file
+            basedir (str): Full path to work or source dir depending on execution mode
+
+        Returns:
+            Full path to GAMSIDE project file or None if operation failed
+        """
+        lst_file_path = os.path.join(basedir, self.lst_file)  # We need the latest from work dir or from source dir
+        main_prgm_path = os.path.join(self.path, self.main_prgm)  # Always get the one from source dir
+        try:
+            with open(prj_file_path, "w") as f:
+                f.write("[PROJECT]\n\n")
+                f.write("[MRUFILES]\n")  # Most Recently Used (MRU)
+                f.write("1=" + lst_file_path + "\n\n")
+                f.write("[OPENWINDOW_1]\n")
+                f.write("FILE0=" + main_prgm_path + "\n")
+                f.write("FILE1=" + lst_file_path + "\n")
+                f.write("FILE2=" + main_prgm_path + "\n")
+                f.write("MAXIM=0\n")
+                f.write("TOP=0\n")
+                f.write("LEFT=0\n")
+                f.write("HEIGHT=600\n")
+                f.write("WIDTH=1000\n")
+        except OSError:
+            raise
+
     @staticmethod
     def load(path, data, settings, logger):
         """Creates a GAMSTool according to a tool specification file.
@@ -348,9 +398,9 @@ class GAMSTool(ToolSpecification):
             return GAMSTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    @property
-    def tool_instance_factory(self):
-        return GAMSToolInstance
+    def create_tool_instance(self, basedir, logger, owner):
+        """See base class."""
+        return GAMSToolInstance(self, basedir, self._settings, logger, owner)
 
 
 class JuliaTool(ToolSpecification):
@@ -376,8 +426,8 @@ class JuliaTool(ToolSpecification):
             name (str): Julia Tool name
             tooltype (str): Tool specification type
             path (str): Path to model main file
-            includes (list): List of files belonging to the tool (relative to 'path').  # TODO: Change to src_files
-            First file in the list is the main Julia program.
+            includes (list): List of files belonging to the tool (relative to 'path')
+                First file in the list is the main Julia program.
             settings (QSettings): Toolbox settings
             logger (LoggerInterface): A logger instance
             description (str): Julia Tool description
@@ -400,8 +450,7 @@ class JuliaTool(ToolSpecification):
             cmdline_args,
             execute_in_work,
         )
-        main_file = includes[0]
-        self.main_dir, self.main_prgm = os.path.split(main_file)
+        self.main_prgm = includes[0]
         self.julia_options = OrderedDict()
         self.return_codes = {0: "Normal return", -1: "Failure"}
 
@@ -424,9 +473,9 @@ class JuliaTool(ToolSpecification):
             return JuliaTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    @property
-    def tool_instance_factory(self):
-        return JuliaToolInstance
+    def create_tool_instance(self, basedir, logger, owner):
+        """See base class."""
+        return JuliaToolInstance(self, basedir, self._settings, logger, owner)
 
 
 class PythonTool(ToolSpecification):
@@ -454,16 +503,16 @@ class PythonTool(ToolSpecification):
             name (str): Python Tool name
             tooltype (str): Tool specification type
             path (str): Path to model main file
-            includes (list): List of files belonging to the tool (relative to 'path').  # TODO: Change to src_files
+            includes (list): List of files belonging to the tool (relative to 'path').
+                First file in the list is the main Python program.
             settings (QSettings): Toolbox settings
             logger (LoggerInterface): A logger instance
-            First file in the list is the main Python program.
             description (str): Python Tool description
             inputfiles (list): List of required data files
             inputfiles_opt (list, optional): List of optional data files (wildcards may be used)
             outputfiles (list, optional): List of output files (wildcards may be used)
             cmdline_args (str, optional): Python tool command line arguments (read from tool definition file)
-            execution_settings (dict): Python kernel spec settings
+            execution_settings (dict, optional): Python kernel spec settings
         """
         super().__init__(
             name,
@@ -479,10 +528,9 @@ class PythonTool(ToolSpecification):
             cmdline_args,
             execute_in_work,
         )
-        main_file = includes[0]
-        self.main_dir, self.main_prgm = os.path.split(main_file)
+        self.main_prgm = includes[0]
         self.python_options = OrderedDict()
-        self.execution_settings = execution_settings
+        self.execution_settings = execution_settings if execution_settings is not None else {}
         self.return_codes = {0: "Normal return", -1: "Failure"}  # Not official
 
     def to_dict(self):
@@ -543,9 +591,9 @@ class PythonTool(ToolSpecification):
             return PythonTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    @property
-    def tool_instance_factory(self):
-        return PythonToolInstance
+    def create_tool_instance(self, basedir, logger, owner):
+        """See base class."""
+        return PythonToolInstance(self, basedir, self._settings, logger, owner)
 
 
 class ExecutableTool(ToolSpecification):
@@ -565,6 +613,8 @@ class ExecutableTool(ToolSpecification):
         outputfiles=None,
         cmdline_args=None,
         execute_in_work=True,
+        execution_settings=None,
+        definition_file_path=None,
     ):
         """
         Args:
@@ -572,8 +622,8 @@ class ExecutableTool(ToolSpecification):
             name (str): Tool name
             tooltype (str): Tool specification type
             path (str): Path to main script file
-            includes (list): List of files belonging to the tool (relative to 'path').  # TODO: Change to src_files
-            First file in the list is the main script file.
+            includes (list): List of files belonging to the tool (relative to 'path')
+                First file in the list is the main program file.
             settings (QSettings): Toolbox settings
             logger (LoggerInterface): A logger instance
             description (str): Tool description
@@ -581,6 +631,9 @@ class ExecutableTool(ToolSpecification):
             inputfiles_opt (list, optional): List of optional data files (wildcards may be used)
             outputfiles (list, optional): List of output files (wildcards may be used)
             cmdline_args (str, optional): Tool command line arguments (read from tool definition file)
+            execution_settings (dict): Settings for executing a (shell) command instead of a file
+            definition_file_path (str): Absolute path to spec definition file. Only used when running a (shell) command
+                in 'source' execution mode
         """
         super().__init__(
             name,
@@ -596,12 +649,54 @@ class ExecutableTool(ToolSpecification):
             cmdline_args,
             execute_in_work,
         )
-        main_file = includes[0]
-        # TODO: This does not do anything because main_file is always just file name
-        self.main_dir, self.main_prgm = os.path.split(main_file)
+        try:
+            self.main_prgm = includes[0]
+        except IndexError:
+            self.main_prgm = None
+        self.execution_settings = execution_settings
+        if definition_file_path and not path:
+            # Set the default execution dir in source execution mode when running a command (no program files)
+            # This part is processed when Tool Specs are loaded at app startup or when Spine Engine loads them
+            # Note: When a Tool Spec is saved in Tool Spec Editor, definition_file_path == None
+            # Default execution dir is the directory of the definition file path
+            self.default_execution_dir, _ = os.path.split(definition_file_path)
         self.options = OrderedDict()
         self.return_codes = {0: "Normal exit", 1: "Error happened"}
 
+    def to_dict(self):
+        """Adds execution settings dict to Executable Tool spec dicts."""
+        d = super().to_dict()
+        d["execution_settings"] = self.execution_settings
+        return d
+
+    def _includes_main_path_relative(self):
+        if not self.path:
+            return None
+        else:
+            return super()._includes_main_path_relative()
+
+    def set_execution_settings(self):
+        """Updates old Executable Tool specifications by adding the
+        default execution settings dict for this specification.
+
+        Returns:
+            void
+        """
+        if not self.execution_settings:
+            d = dict()
+            d["cmd"] = ""
+            d["shell"] = ""
+            self.execution_settings = d
+        else:
+            # Make sure that required keys are included (for debugging)
+            if not isinstance(self.execution_settings, dict):
+                logging.error("self.execution_settings is not a dict")
+                return
+            if "cmd" not in self.execution_settings.keys():
+                logging.error("self.execution_settings error 1")
+            elif "shell" not in self.execution_settings.keys():
+                logging.error("self.execution_settings error 2")
+
     @staticmethod
     def load(path, data, settings, logger):
         """Creates an ExecutableTool according to a tool specification file.
@@ -617,10 +712,13 @@ class ExecutableTool(ToolSpecification):
         """
         kwargs = ExecutableTool.check_definition(data, logger)
         if kwargs is not None:
+            # Add definition_file_path. 'data' does not have definition_file_path
+            # when creating a new Tool spec or editing an existing one in Tool Spec Editor.
+            kwargs["definition_file_path"] = data.get("definition_file_path", None)
             # Return an executable model instance
             return ExecutableTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    @property
-    def tool_instance_factory(self):
-        return ExecutableToolInstance
+    def create_tool_instance(self, basedir, logger, owner):
+        """See base class."""
+        return ExecutableToolInstance(self, basedir, self._settings, logger, owner)
